"""A collection of Callable classes to be used by Processors and TransitionPredicates.
"""
from gabrieltool.statemachine.callable_zoo.base import record_kwargs, CallableBase, Null  # noqa: F401
from gabrieltool.statemachine.callable_zoo import processor_zoo  # noqa: F401
from gabrieltool.statemachine.callable_zoo import predicate_zoo  # noqa: F401
