"""Library to build application logic as a Finite State Machine.

_pb2 modules are generated python modules by Protobuf for serialization.
See the proto directory for the serialization formats.
"""

from gabrieltool.statemachine.callable_zoo import processor_zoo, predicate_zoo  # noqa: F401
