# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: wca-state-machine.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='wca-state-machine.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x17wca-state-machine.proto\"\xcb\x01\n\x13TransitionPredicate\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rcallable_name\x18\x02 \x01(\t\x12\x41\n\x0f\x63\x61llable_kwargs\x18\x03 \x03(\x0b\x32(.TransitionPredicate.CallableKwargsEntry\x12\x15\n\rcallable_args\x18\x04 \x01(\t\x1a\x35\n\x13\x43\x61llableKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\"H\n\x0bInstruction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x61udio\x18\x02 \x01(\t\x12\r\n\x05image\x18\x03 \x01(\x0c\x12\r\n\x05video\x18\x04 \x01(\x0c\"{\n\nTransition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12(\n\npredicates\x18\x02 \x03(\x0b\x32\x14.TransitionPredicate\x12!\n\x0binstruction\x18\x03 \x01(\x0b\x32\x0c.Instruction\x12\x12\n\nnext_state\x18\x04 \x01(\t\"\xb7\x01\n\tProcessor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rcallable_name\x18\x02 \x01(\t\x12\x37\n\x0f\x63\x61llable_kwargs\x18\x03 \x03(\x0b\x32\x1e.Processor.CallableKwargsEntry\x12\x15\n\rcallable_args\x18\x04 \x01(\t\x1a\x35\n\x13\x43\x61llableKwargsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\"W\n\x05State\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1e\n\nprocessors\x18\x02 \x03(\x0b\x32\n.Processor\x12 \n\x0btransitions\x18\x03 \x03(\x0b\x32\x0b.Transition\"\xa3\x01\n\x0cStateMachine\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x16\n\x06states\x18\x02 \x03(\x0b\x32\x06.State\x12)\n\x06\x61ssets\x18\x03 \x03(\x0b\x32\x19.StateMachine.AssetsEntry\x12\x13\n\x0bstart_state\x18\x04 \x01(\t\x1a-\n\x0b\x41ssetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x62\x06proto3')
)




_TRANSITIONPREDICATE_CALLABLEKWARGSENTRY = _descriptor.Descriptor(
  name='CallableKwargsEntry',
  full_name='TransitionPredicate.CallableKwargsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='TransitionPredicate.CallableKwargsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='TransitionPredicate.CallableKwargsEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=231,
)

_TRANSITIONPREDICATE = _descriptor.Descriptor(
  name='TransitionPredicate',
  full_name='TransitionPredicate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='TransitionPredicate.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_name', full_name='TransitionPredicate.callable_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_kwargs', full_name='TransitionPredicate.callable_kwargs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_args', full_name='TransitionPredicate.callable_args', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TRANSITIONPREDICATE_CALLABLEKWARGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=231,
)


_INSTRUCTION = _descriptor.Descriptor(
  name='Instruction',
  full_name='Instruction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Instruction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='audio', full_name='Instruction.audio', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image', full_name='Instruction.image', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='video', full_name='Instruction.video', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=233,
  serialized_end=305,
)


_TRANSITION = _descriptor.Descriptor(
  name='Transition',
  full_name='Transition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Transition.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='predicates', full_name='Transition.predicates', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='instruction', full_name='Transition.instruction', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next_state', full_name='Transition.next_state', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=307,
  serialized_end=430,
)


_PROCESSOR_CALLABLEKWARGSENTRY = _descriptor.Descriptor(
  name='CallableKwargsEntry',
  full_name='Processor.CallableKwargsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Processor.CallableKwargsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Processor.CallableKwargsEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=178,
  serialized_end=231,
)

_PROCESSOR = _descriptor.Descriptor(
  name='Processor',
  full_name='Processor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Processor.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_name', full_name='Processor.callable_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_kwargs', full_name='Processor.callable_kwargs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='callable_args', full_name='Processor.callable_args', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PROCESSOR_CALLABLEKWARGSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=433,
  serialized_end=616,
)


_STATE = _descriptor.Descriptor(
  name='State',
  full_name='State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='State.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processors', full_name='State.processors', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='transitions', full_name='State.transitions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=618,
  serialized_end=705,
)


_STATEMACHINE_ASSETSENTRY = _descriptor.Descriptor(
  name='AssetsEntry',
  full_name='StateMachine.AssetsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='StateMachine.AssetsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='StateMachine.AssetsEntry.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=826,
  serialized_end=871,
)

_STATEMACHINE = _descriptor.Descriptor(
  name='StateMachine',
  full_name='StateMachine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='StateMachine.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='states', full_name='StateMachine.states', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='assets', full_name='StateMachine.assets', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_state', full_name='StateMachine.start_state', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_STATEMACHINE_ASSETSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=708,
  serialized_end=871,
)

_TRANSITIONPREDICATE_CALLABLEKWARGSENTRY.containing_type = _TRANSITIONPREDICATE
_TRANSITIONPREDICATE.fields_by_name['callable_kwargs'].message_type = _TRANSITIONPREDICATE_CALLABLEKWARGSENTRY
_TRANSITION.fields_by_name['predicates'].message_type = _TRANSITIONPREDICATE
_TRANSITION.fields_by_name['instruction'].message_type = _INSTRUCTION
_PROCESSOR_CALLABLEKWARGSENTRY.containing_type = _PROCESSOR
_PROCESSOR.fields_by_name['callable_kwargs'].message_type = _PROCESSOR_CALLABLEKWARGSENTRY
_STATE.fields_by_name['processors'].message_type = _PROCESSOR
_STATE.fields_by_name['transitions'].message_type = _TRANSITION
_STATEMACHINE_ASSETSENTRY.containing_type = _STATEMACHINE
_STATEMACHINE.fields_by_name['states'].message_type = _STATE
_STATEMACHINE.fields_by_name['assets'].message_type = _STATEMACHINE_ASSETSENTRY
DESCRIPTOR.message_types_by_name['TransitionPredicate'] = _TRANSITIONPREDICATE
DESCRIPTOR.message_types_by_name['Instruction'] = _INSTRUCTION
DESCRIPTOR.message_types_by_name['Transition'] = _TRANSITION
DESCRIPTOR.message_types_by_name['Processor'] = _PROCESSOR
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['StateMachine'] = _STATEMACHINE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransitionPredicate = _reflection.GeneratedProtocolMessageType('TransitionPredicate', (_message.Message,), dict(

  CallableKwargsEntry = _reflection.GeneratedProtocolMessageType('CallableKwargsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSITIONPREDICATE_CALLABLEKWARGSENTRY,
    __module__ = 'wca_state_machine_pb2'
    # @@protoc_insertion_point(class_scope:TransitionPredicate.CallableKwargsEntry)
    ))
  ,
  DESCRIPTOR = _TRANSITIONPREDICATE,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:TransitionPredicate)
  ))
_sym_db.RegisterMessage(TransitionPredicate)
_sym_db.RegisterMessage(TransitionPredicate.CallableKwargsEntry)

Instruction = _reflection.GeneratedProtocolMessageType('Instruction', (_message.Message,), dict(
  DESCRIPTOR = _INSTRUCTION,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:Instruction)
  ))
_sym_db.RegisterMessage(Instruction)

Transition = _reflection.GeneratedProtocolMessageType('Transition', (_message.Message,), dict(
  DESCRIPTOR = _TRANSITION,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:Transition)
  ))
_sym_db.RegisterMessage(Transition)

Processor = _reflection.GeneratedProtocolMessageType('Processor', (_message.Message,), dict(

  CallableKwargsEntry = _reflection.GeneratedProtocolMessageType('CallableKwargsEntry', (_message.Message,), dict(
    DESCRIPTOR = _PROCESSOR_CALLABLEKWARGSENTRY,
    __module__ = 'wca_state_machine_pb2'
    # @@protoc_insertion_point(class_scope:Processor.CallableKwargsEntry)
    ))
  ,
  DESCRIPTOR = _PROCESSOR,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:Processor)
  ))
_sym_db.RegisterMessage(Processor)
_sym_db.RegisterMessage(Processor.CallableKwargsEntry)

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), dict(
  DESCRIPTOR = _STATE,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:State)
  ))
_sym_db.RegisterMessage(State)

StateMachine = _reflection.GeneratedProtocolMessageType('StateMachine', (_message.Message,), dict(

  AssetsEntry = _reflection.GeneratedProtocolMessageType('AssetsEntry', (_message.Message,), dict(
    DESCRIPTOR = _STATEMACHINE_ASSETSENTRY,
    __module__ = 'wca_state_machine_pb2'
    # @@protoc_insertion_point(class_scope:StateMachine.AssetsEntry)
    ))
  ,
  DESCRIPTOR = _STATEMACHINE,
  __module__ = 'wca_state_machine_pb2'
  # @@protoc_insertion_point(class_scope:StateMachine)
  ))
_sym_db.RegisterMessage(StateMachine)
_sym_db.RegisterMessage(StateMachine.AssetsEntry)


_TRANSITIONPREDICATE_CALLABLEKWARGSENTRY._options = None
_PROCESSOR_CALLABLEKWARGSENTRY._options = None
_STATEMACHINE_ASSETSENTRY._options = None
# @@protoc_insertion_point(module_scope)
